@@ -28,6 +28,7 @@
 ;; Feature masks:
 (define droppable-feature 1)
 (define scores-disabled 2)
+(define dealable-feature 4)
 
 (define (reset-features) (set-feature-word! 0))
 
@@ -470,21 +471,8 @@
 (define (set-cards! slot-id new_cards)
   (set-cards-c! slot-id new_cards))
 
-; Because of the way the cards are passed back and forth to
-; the C code it is possible for references to the cards to be
-; lost. (e.g. when the cards are being moved the list is kept
-; entirely on the C side.) To avoid premature garbage collection 
-; we keep a list of references to the cards.
-;
-; Note that this approach leaks memory since sol.scm is only 
-; re-evaluated and the list emptied when the game has changed.
-; The loss is probably about 1600 bytes per game.
-(define card-ref-list '())
-
 (define (make-card value suit)
-  (let ((new-card (list value suit #f)))
-    (set! card-ref-list (cons new-card card-ref-list))
-    new-card))
+  (list value suit #f))
 
 (define (make-standard-deck-list-ace-high value suit)
   (if (eq? ace value)
