@@ -42,7 +42,6 @@ import glchess.ui
 import glchess.chess.board
 import dialogs
 import chessview
-import network
 
 # Mark all windows with our icon
 gtk.window_set_default_icon_name(ICON_NAME)
@@ -358,8 +357,7 @@ class GtkUI(glchess.ui.UI):
 
     # TODO
     __joinGameDialogs  = None
-    __networkGames     = None
-    
+
     __defaultWhiteAI   = None
     __defaultBlackAI   = None
 
@@ -381,7 +379,6 @@ class GtkUI(glchess.ui.UI):
                                properties={gnome.PARAM_APP_DATADIR: APP_DATA_DIR})
 
         self.feedback = feedback
-        self.__networkGames = {}
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
 
@@ -430,12 +427,6 @@ class GtkUI(glchess.ui.UI):
             icon = None
         iter = self.__playerModel.append()
         self.__playerModel.set(iter, 0, '', 1, icon, 2, _('Human'))
-        #try:
-        #    icon = iconTheme.load_icon(gtk.STOCK_NETWORK, 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        #except gobject.GError:
-        #    icon = None
-        #iter = self.__playerModel.append()
-        #self.__playerModel.set(iter, 0, None, 1, icon, 2, _('Network'))
         
         self.__aiWindow = AIWindow(self._gui.get_widget('ai_notebook'))
 
@@ -465,8 +456,7 @@ class GtkUI(glchess.ui.UI):
                     'show_3d', 'show_comments', 'show_numbering',
                     'show_move_hints',
                     'width', 'height',
-                    'move_format', 'promotion_type', 'board_view',
-                    'enable_networking']:
+                    'move_format', 'promotion_type', 'board_view']:
             glchess.config.watch(key, self.__applyConfig)
 
     # Public methods
@@ -558,8 +548,7 @@ class GtkUI(glchess.ui.UI):
         # Load configuration
         for name in ['show_toolbar', 'show_history', 'show_3d',
                      'show_comments', 'show_numbering', 'show_move_hints',
-                     'move_format', 'promotion_type', 'board_view', 'maximised',
-                     'enable_networking']:
+                     'move_format', 'promotion_type', 'board_view', 'maximised']:
             try:
                 value = glchess.config.get(name)
             except glchess.config.Error:
@@ -584,34 +573,6 @@ class GtkUI(glchess.ui.UI):
         """Extends glchess.ui.UI"""
         dialogs.GtkNewGameDialog(self, self.__playerModel, game)
 
-    def addNetworkDialog(self, feedback):
-        """Extends glchess.ui.UI"""
-        # Create the dialog
-        dialog = network.GtkNetworkGameDialog(self, feedback, self.__playerModel)
-        self.__joinGameDialogs.append(dialog)
-        
-        # Add the detected games into the dialog
-        #for (game, name) in self.__networkGames.iteritems():
-        #    dialog.addNetworkGame(name, game)
-        
-        return dialog
-                                 
-    def addNetworkGame(self, name, game):
-        """Extends glchess.ui.UI"""
-        self.__networkGames[game] = name
-        
-        # Update the open dialogs
-        for dialog in self.__joinGameDialogs:
-            dialog.addNetworkGame(name, game)
-
-    def removeNetworkGame(self, game):
-        """Extends glchess.ui.UI"""
-        self.__networkGames.pop(game)
-
-        # Update the open dialogs
-        for dialog in self.__joinGameDialogs:
-            dialog.removeNetworkGame(game)
-        
     def close(self):
         """Extends glchess.ui.UI"""
         # Save the window size
@@ -687,16 +648,6 @@ class GtkUI(glchess.ui.UI):
                 menu.set_active(False)
                 toolbar.hide()
                 
-        elif name == 'enable_networking':
-            menuItem = self.__getWidget('menu_play_online_item')
-            toolbarButton = self.__getWidget('play_online_button')
-            if value is True:
-                menuItem.show()
-                toolbarButton.show()
-            else:
-                menuItem.hide()
-                toolbarButton.hide()
-            
         # Show/hide the history
         elif name == 'show_history':
             box = self.__getWidget('navigation_box')
@@ -1028,10 +979,6 @@ class GtkUI(glchess.ui.UI):
         """Gtk+ callback"""
         dialogs.GtkNewGameDialog(self, self.__playerModel)
 
-    def _on_join_game_button_clicked(self, widget):
-        """Gtk+ callback"""
-        self.feedback.onNewNetworkGame()
-
     def _on_open_game_button_clicked(self, widget):
         """Gtk+ callback"""
         dialogs.GtkLoadGameDialog(self)
