@@ -2,10 +2,12 @@
 
 /*
  * Color lines for GNOME
- * (c) 1999 Free Software Foundation
+ * Copyright © 1999 Free Software Foundation
  * Authors: Robert Szokovacs <szo@appaloosacorp.hu>
  *          Szabolcs Ban <shooby@gnome.hu>
  *
+ * Copyright © 2007 Christian Persch
+ *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -22,15 +24,15 @@
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
+#include <stdlib.h>    
 #include <math.h>
-#include <gnome.h>
+
+#include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gconf/gconf-client.h>
+#include <gdk/gdkkeysyms.h>
 #include <games-scores.h>
 #include <games-scores-dialog.h>
 #include <games-frame.h>
@@ -38,24 +40,29 @@
 #include <games-gridframe.h>
 #include <games-preimage.h>
 #include <games-stock.h>
+#include <games-conf.h>
+
+#ifdef HAVE_GNOME
+#include <gnome.h>
+#endif
+
 #include "glines.h"
 
-#define KEY_DIR "/apps/glines"
-#define KEY_BACKGROUND_COLOR "/apps/glines/preferences/background_color"
-#define KEY_BALL_THEME "/apps/glines/preferences/ball_theme"
-#define KEY_MOVE_TIMEOUT "/apps/glines/preferences/move_timeout"
-#define KEY_SAVED_SCORE "/apps/glines/saved/score"
-#define KEY_SAVED_FIELD "/apps/glines/saved/field"
-#define KEY_SAVED_PREVIEW "/apps/glines/saved/preview"
-#define KEY_WIDTH "/apps/glines/saved/width"
-#define KEY_HEIGHT "/apps/glines/saved/height"
-#define KEY_MAXIMIZED "/apps/glines/saved/maximized"
-#define KEY_FULLSCREEN "/apps/glines/saved/fullscreen"
-#define KEY_SIZE "/apps/glines/preferences/size"
+#define KEY_PREFERENCES_GROUP "preferences"
+#define KEY_BACKGROUND_COLOR  "background_color"
+#define KEY_BALL_THEME        "ball_theme"
+#define KEY_MOVE_TIMEOUT      "move_timeout"
+#define KEY_SIZE              "size"
+
+#define KEY_SAVED_GROUP   "saved"
+#define KEY_SAVED_SCORE   "score"
+#define KEY_SAVED_FIELD   "field"
+#define KEY_SAVED_PREVIEW "preview"
 
 #define MAXNPIECES 10
 #define MAXFIELDSIZE 30
 #define DEFAULT_GAME_SIZE MEDIUM
+#define DEFAULT_BALL_THEME "balls.svg"
 
 enum {
   UNSET = 0,
@@ -66,104 +73,92 @@ enum {
 };
 
 /* Keep these in sync with the enum above. */
-const
-gint field_sizes[MAX_SIZE][4] = { {-1, -1, -1, -1},	/* This is a dummy entry. */
-{7, 7, 5, 3},			/* SMALL */
-{9, 9, 7, 3},			/* MEDIUM */
-{20, 15, 7, 7}
-};				/* LARGE */
-
-const
-gchar *scorenames[] = { N_("Small"),
-  N_("glines|Medium"),
-  N_("Large")
+static const gint field_sizes[MAX_SIZE][4] = {
+  {-1, -1, -1, -1}, /* This is a dummy entry. */
+  {7, 7, 5, 3},     /* SMALL */
+  {9, 9, 7, 3},     /* MEDIUM */
+  {20, 15, 7, 7}    /* LARGE */
 };
 
-static const GamesScoresCategory scorecats[] = { {"Small", N_("Small")},
-{"Medium", N_("glines|Medium")},
-{"Large", N_("Large")},
-GAMES_SCORES_LAST_CATEGORY
+static const GamesScoresCategory scorecats[] = {
+  { "Small",  N_("Small") },
+  { "Medium", N_("glines|Medium") },
+  { "Large",  N_("Large") },
+  GAMES_SCORES_LAST_CATEGORY
 };
 
-static const GamesScoresDescription scoredesc = { scorecats,
+static const GamesScoresDescription scoredesc = {
+  scorecats,
   "Small",
   "glines",
   GAMES_SCORES_STYLE_PLAIN_DESCENDING
 };
 
-GamesScores *highscores;
+static GamesScores *highscores;
 
+static gint hfieldsize;
+static gint vfieldsize;
+static gint ncolors;
+static gint npieces;
+static gint game_size = UNSET;
+static gboolean pref_dialog_done = FALSE;
 
+static GRand *rgen;
 
-gint hfieldsize;
-gint vfieldsize;
-gint ncolors;
-gint npieces;
-gint game_size = UNSET;
-gboolean pref_dialog_done = FALSE;
-gboolean is_maximized = FALSE;
-gboolean is_fullscreen = FALSE;
-
-GRand *rgen;
-
-GConfClient *conf_client = NULL;
-
-GtkWidget *draw_area;
-GtkWidget *app, *statusbar, *pref_dialog, *gridframe;
-GtkWidget *preview_widgets[MAXNPIECES];
-GtkWidget *menubar;
-GtkWidget *scoreitem;
-GtkAction *fullscreen_action;
-GtkAction *leavefullscreen_action;
+static GtkWidget *draw_area;
+static GtkWidget *app, *statusbar, *pref_dialog, *gridframe;
+static GtkWidget *preview_widgets[MAXNPIECES];
+static GtkWidget *menubar;
+static GtkWidget *scoreitem;
+static GtkAction *fullscreen_action;
+static GtkAction *leavefullscreen_action;
 
 /* These keep track of what we put in the main table so we
  * can reshuffle them when we change the field size. */
-GtkWidget *table;
-GtkWidget *top_pane;
-GtkWidget *bottom_pane;
+static GtkWidget *table;
+static GtkWidget *top_pane;
+static GtkWidget *bottom_pane;
 
-field_props field[MAXFIELDSIZE * MAXFIELDSIZE];
+static field_props field[MAXFIELDSIZE * MAXFIELDSIZE];
 
 /* Pre-rendering image data prepared from file. */
-GamesPreimage *ball_preimage = NULL;
+static GamesPreimage *ball_preimage = NULL;
 /* The tile images with balls rendered on them. */
-GdkPixmap *ball_pixmap = NULL;
+static GdkPixmap *ball_pixmap = NULL;
 /* The balls rendered to a size appropriate for the preview. */
-GdkPixmap *preview_pixmaps[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+static GdkPixmap *preview_pixmaps[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 
 /* A pixmap of a blank tile. */
-GdkPixmap *blank_pixmap = NULL;
-GdkPixmap *blank_preview_pixmap = NULL;
+static GdkPixmap *blank_pixmap = NULL;
+static GdkPixmap *blank_preview_pixmap = NULL;
 
-GtkWidget *fast_moves_toggle_button = NULL;
+static GtkWidget *fast_moves_toggle_button = NULL;
 
-GamesFileList *theme_file_list = NULL;
+static GamesFileList *theme_file_list = NULL;
 
-int active = -1;
-int target = -1;
-int inmove = 0;
-guint score = 0;
-int cursor_x = MAXFIELDSIZE / 2;
-int cursor_y = MAXFIELDSIZE / 2;
-gboolean show_cursor = FALSE;
+static int active = -1;
+static int target = -1;
+static int inmove = 0;
+static guint score = 0;
+static int cursor_x = MAXFIELDSIZE / 2;
+static int cursor_y = MAXFIELDSIZE / 2;
+static gboolean show_cursor = FALSE;
 
-int boxsize;
+static int boxsize;
 
 /* The width and height of the main window. */
-#define MIN_WIDTH  190
-#define MIN_HEIGHT 240
-int width;
-int height;
+#define DEFAULT_WIDTH  320
+#define DEFAULT_HEIGHT 400
 
-int preview_height = 0;
-int preview_width = 0;
+static int preview_height = 0;
+static int preview_width = 0;
 
-int move_timeout = 100;
-int animate_id = 0;
-int preview[MAXNPIECES];
-char *ball_filename;
-GtkWidget *scorelabel;
-scoretable sctab[] =
+static int move_timeout = 100;
+static int animate_id = 0;
+static int preview[MAXNPIECES];
+static char *ball_filename;
+static GtkWidget *scorelabel;
+static scoretable sctab[] =
   { {5, 10}, {6, 12}, {7, 18}, {8, 28}, {9, 42}, {10, 82}, {11, 108}, {12,
 								       138},
   {13, 172}, {14, 210}, {0, 0} };
@@ -172,11 +167,9 @@ static struct {
   GdkColor color;
   gchar *name;
   gint set;
-} backgnd = {
-  {
-0, 0, 0, 0}, NULL, 0};
+} backgnd = { { 0, 0, 0, 0}, NULL, 0 };
 
-gchar *warning_message = NULL;
+static gchar *warning_message = NULL;
 
 static void
 set_statusbar_message (gchar * message)
@@ -215,47 +208,37 @@ show_image_warning (gchar * message)
   gtk_widget_destroy (dialog);
 }
 
-static void
-load_image (gchar * fname, GamesPreimage ** preimage)
+static GamesPreimage *
+load_image (gchar * fname)
 {
-  gchar *tmp, *fn = NULL;
-  GamesPreimage *tmp_preimage;
+  GamesPreimage *preimage;
+  gchar *path;
+  GError *error = NULL;
 
-  tmp = g_build_filename ("glines", fname, NULL);
-
-  fn =
-    gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp),
-			       FALSE, NULL);
-  g_free (tmp);
-
-
-  if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+  path = g_build_filename (PIXMAPDIR, fname, NULL);
+  if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
     warning_message = g_strdup_printf (_("Unable to locate file:\n%s\n\n"
 					 "The default theme will be loaded instead."),
-				       fn);
+				       fname);
 
-    tmp = g_build_filename ("glines", "balls.svg", NULL);
-    fn =
-      gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp),
-				 FALSE, NULL);
-    g_free (tmp);
-
-    if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+    path = g_build_filename (PIXMAPDIR, "balls.svg", NULL);
+    if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
       g_free (warning_message);
       warning_message = g_strdup_printf (_("Unable to locate file:\n%s\n\n"
 					   "Please check that Five or More is installed correctly."),
-					 fn);
+					 fname);
     }
   }
 
-  tmp_preimage = games_preimage_new_from_file (fn, NULL);
-  g_free (fn);
+  preimage = games_preimage_new_from_file (path, &error);
+  g_free (path);
 
+  if (error) {
+    warning_message = g_strdup (error->message);
+    g_error_free (error);
+  }
 
-  if (*preimage)
-    g_object_unref (*preimage);
-
-  *preimage = tmp_preimage;
+  return preimage;
 }
 
 /* The main table has to be layed out differently depending on the
@@ -291,7 +274,7 @@ refresh_pixmaps (void)
 
   if (ball_preimage) {
     ball_pixbuf = games_preimage_render (ball_preimage, 4 * boxsize,
-					 7 * boxsize, NULL);
+					 7 * boxsize);
 
     /* Handle rendering problems. */
     if (!ball_pixbuf) {
@@ -335,7 +318,7 @@ refresh_pixmaps (void)
 static void
 refresh_preview_pixmaps (void)
 {
-  int i;
+  guint i;
   GdkPixbuf *scaled = NULL;
   GtkWidget *widget = preview_widgets[0];
 
@@ -348,13 +331,13 @@ refresh_preview_pixmaps (void)
    * set them as the background for each widget in the preview array.
    * This code assumes that each preview window is identical. */
 
-  if (preview_pixmaps[0])
-    for (i = 0; i < 7; i++)
+  for (i = 0; i < 7; i++)
+    if (preview_pixmaps[i])
       g_object_unref (preview_pixmaps[i]);
 
   if (ball_preimage)
     scaled = games_preimage_render (ball_preimage, 4 * preview_width,
-				    7 * preview_height, NULL);
+				    7 * preview_height);
 
   if (!scaled) {
     scaled = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
@@ -469,7 +452,7 @@ init_preview (void)
 void
 draw_preview (void)
 {
-  int i;
+  guint i;
 
   for (i = 0; i < MAXNPIECES; i++) {
 
@@ -500,7 +483,7 @@ show_scores (gint pos, gboolean new_game)
   if (dialog == NULL) {
     dialog = games_scores_dialog_new (GTK_WINDOW (app), highscores, _("GNOME Five or More"));
     games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG
-						  (dialog), _("Board Size:"));
+						  (dialog), _("_Board size:"));
   }
 
   if (pos > 0) {
@@ -1209,7 +1192,7 @@ game_top_ten_callback (GtkAction * action, gpointer data)
   show_scores (0, FALSE);
 }
 
-static int
+static void
 game_about_callback (GtkAction * action, gpointer * data)
 {
   const gchar *authors[] = { "Robert Szokovacs <szo@appaloosacorp.hu>",
@@ -1223,24 +1206,28 @@ game_about_callback (GtkAction * action, gpointer * data)
   gchar *license = games_get_license (_("Five or More"));
 
   gtk_show_about_dialog (GTK_WINDOW (app),
-			 "name", _("Five or More"),
+#if GTK_CHECK_VERSION (2, 11, 0)
+                         "program-name", _("Five or More"),
+#else
+                         "name", _("Five or More"),
+#endif
 			 "version", VERSION,
-			 "comments",
-			 _("GNOME port of the once-popular Color Lines game"),
+			 "comments", _("GNOME port of the once-popular Color Lines game.\n\nFive or More is a part of GNOME Games."),
 			 "copyright",
 			 "Copyright \xc2\xa9 1997-2007 Free Software Foundation, Inc.",
-			 "license", license, "authors", authors,
-			 "documenters", documenters, "translator_credits",
-			 _("translator-credits"), "logo-icon-name",
-			 "gnome-five-or-more", "website",
-			 "http://www.gnome.org/projects/gnome-games/",
+			 "license", license,
+                         "authors", authors,
+			 "documenters", documenters,
+			 "translator-credits", _("translator-credits"),
+			 "logo-icon-name", "gnome-glines",
+			 "website", "http://www.gnome.org/projects/gnome-games/",
+                         "website-label", _("GNOME Games web site"),
 			 "wrap-license", TRUE, NULL);
   g_free (license);
-  return TRUE;
 }
 
 static void
-set_backgnd_color (gchar * str)
+set_backgnd_color (const gchar * str)
 {
   GdkColormap *colormap;
   GtkStyle *widget_style, *temp_style;
@@ -1269,30 +1256,6 @@ set_backgnd_color (gchar * str)
   gtk_widget_set_style (draw_area, temp_style);
 }
 
-static void
-bg_color_changed_cb (GConfClient * client,
-		     guint cnxn_id, GConfEntry * entry, gpointer user_data)
-{
-  gchar *color;
-
-  color = gconf_client_get_string (client, KEY_BACKGROUND_COLOR, NULL);
-  if (color != NULL)
-    set_backgnd_color (color);
-}
-
-static void
-bg_color_callback (GtkWidget * widget, gpointer data)
-{
-  static char *tmp = "";
-  GdkColor c;
-
-  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &c);
-
-  tmp = g_strdup_printf ("#%04x%04x%04x", c.red, c.green, c.blue);
-
-  gconf_client_set_string (conf_client, KEY_BACKGROUND_COLOR, tmp, NULL);
-}
-
 static void
 set_sizes (gint size)
 {
@@ -1303,67 +1266,117 @@ set_sizes (gint size)
   game_size = size;
   games_scores_set_category (highscores, scorecats[size - 1].key);
 
-  gconf_client_set_int (conf_client, KEY_SIZE, size, NULL);
+  games_conf_set_integer (KEY_PREFERENCES_GROUP, KEY_SIZE, size);
 
   if (gridframe)
     games_grid_frame_set (GAMES_GRID_FRAME (gridframe),
-			  hfieldsize, vfieldsize);
+                          hfieldsize, vfieldsize);
+}
+
+static void
+load_theme ()
+{
+  if (ball_preimage)
+    g_object_unref (ball_preimage);
+  ball_preimage = load_image (ball_filename);
+
+  refresh_pixmaps ();
+  refresh_preview_pixmaps ();
+}
+
+static void
+conf_value_changed_cb (GamesConf *conf,
+                       const char *group,
+                       const char *key,
+                       gpointer user_data)
+{
+  if (!group || strcmp (group, KEY_PREFERENCES_GROUP) != 0)
+    return;
+
+  if (strcmp (key, KEY_BACKGROUND_COLOR) == 0) {
+    gchar *color;
+
+    color = games_conf_get_string (KEY_PREFERENCES_GROUP, KEY_BACKGROUND_COLOR, NULL);
+    if (color != NULL) {
+      set_backgnd_color (color);
+      g_free (color);
+    }
+  } else if (strcmp (key, KEY_BALL_THEME) == 0) {
+    gchar *theme_tmp = NULL;
+
+    theme_tmp = games_conf_get_string (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, NULL);
+    if (theme_tmp) {
+      if (strcmp (theme_tmp, ball_filename) != 0) {
+        g_free (ball_filename);
+        ball_filename = theme_tmp;
+        load_theme ();
+        refresh_screen ();
+      } else
+        g_free (theme_tmp);
+    }
+    /* FIXME apply in the prefs dialog GUI */
+  } else if (strcmp (key, KEY_MOVE_TIMEOUT) == 0) {
+    gint timeout_tmp;
+
+    timeout_tmp = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, NULL);
+    timeout_tmp = CLAMP (timeout_tmp, 1, 1000);
+    if (timeout_tmp != move_timeout)
+      move_timeout = timeout_tmp;
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fast_moves_toggle_button),
+                                  (move_timeout == 10));
+
+  } else if (strcmp (key, KEY_SIZE) == 0) {
+    gint size_tmp;
+    size_tmp = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_SIZE, NULL);
+
+    if (size_tmp != game_size) {
+      set_sizes (size_tmp);
+      relay_table ();
+      reset_game ();
+      start_game ();
+    }
+  }
+}
+
+static void
+bg_color_callback (GtkWidget * widget, gpointer data)
+{
+  GdkColor c;
+  char str[64];
+
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &c);
+
+  g_snprintf (str, sizeof (str), "#%04x%04x%04x", c.red, c.green, c.blue);
+
+  games_conf_set_string (KEY_PREFERENCES_GROUP, KEY_BACKGROUND_COLOR, str);
 }
 
 static void
 size_callback (GtkWidget * widget, gpointer data)
 {
   if (pref_dialog_done)
-    gconf_client_set_int (conf_client, KEY_SIZE, GPOINTER_TO_INT (data),
-			  NULL);
-}
-
-static void
-size_changed_cb (GConfClient * client,
-		 guint cnxn_id, GConfEntry * entry, gpointer user_data)
-{
-  gint size_tmp;
-  size_tmp = gconf_client_get_int (conf_client, KEY_SIZE, NULL);
-
-  if (size_tmp != game_size) {
-    set_sizes (size_tmp);
-    relay_table ();
-    reset_game ();
-    start_game ();
-  }
-}
-
-static void
-load_theme ()
-{
-  load_image (ball_filename, &ball_preimage);
-  refresh_pixmaps ();
-  refresh_preview_pixmaps ();
+    games_conf_set_integer (KEY_PREFERENCES_GROUP, KEY_SIZE, GPOINTER_TO_INT (data));
 }
 
 static void
 set_selection (GtkWidget * widget, char *data)
 {
-  gchar *entry;
+  const gchar *entry;
 
   entry = games_file_list_get_nth (theme_file_list,
 				   gtk_combo_box_get_active (GTK_COMBO_BOX
 							     (widget)));
-  gconf_client_set_string (conf_client, KEY_BALL_THEME, entry, NULL);
+  games_conf_set_string (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, entry);
 }
 
 static GtkWidget *
 fill_menu (void)
 {
-  gchar *dname =
-    gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-			       ("glines"), FALSE, NULL);
-
   if (theme_file_list)
     g_object_unref (theme_file_list);
 
-  theme_file_list = games_file_list_new_images (dname, NULL);
-  g_free (dname);
+  theme_file_list = games_file_list_new_images (PIXMAPDIR, NULL);
   games_file_list_transform_basename (theme_file_list);
 
   return games_file_list_create_widget (theme_file_list, ball_filename,
@@ -1376,20 +1389,17 @@ set_fast_moves_callback (GtkWidget * widget, gpointer * data)
 {
   gboolean is_on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
   gint timeout = is_on ? 10 : 100;
-  gconf_client_set_int (conf_client, KEY_MOVE_TIMEOUT, timeout, NULL);
+  games_conf_set_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, timeout);
 }
 
 static void
 set_fullscreen_actions (gboolean fullscreen)
 {
-  is_fullscreen = fullscreen != FALSE;
-  gconf_client_set_bool (conf_client, KEY_FULLSCREEN, is_fullscreen, NULL);
+  gtk_action_set_sensitive (leavefullscreen_action, fullscreen);
+  gtk_action_set_visible (leavefullscreen_action, fullscreen);
 
-  gtk_action_set_sensitive (leavefullscreen_action, is_fullscreen);
-  gtk_action_set_visible (leavefullscreen_action, is_fullscreen);
-
-  gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
-  gtk_action_set_visible (fullscreen_action, !is_fullscreen);
+  gtk_action_set_sensitive (fullscreen_action, !fullscreen);
+  gtk_action_set_visible (fullscreen_action, !fullscreen);
 }
 
 static void
@@ -1404,21 +1414,9 @@ fullscreen_callback (GtkAction * action)
 static gboolean
 window_state_callback (GtkWidget * widget, GdkEventWindowState * event)
 {
-  gboolean maximized_changed, fullscreen_changed;
-  gboolean new_fullscreen;
-
-  maximized_changed = event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED;
-  fullscreen_changed = event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN;
-  if (!maximized_changed && !fullscreen_changed)
-    return FALSE;
-
-  is_maximized = (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) != 0;
-  new_fullscreen = (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
-
-  if (!fullscreen_changed)
-    gconf_client_set_bool (conf_client, KEY_MAXIMIZED, is_maximized, NULL);
-
-  set_fullscreen_actions (new_fullscreen);
+  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
+    set_fullscreen_actions ((event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0);
+  }
 
   return FALSE;
 }
@@ -1445,9 +1443,9 @@ game_props_callback (void)
 					       GTK_STOCK_CLOSE,
 					       GTK_RESPONSE_CLOSE, NULL);
     gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
-    g_signal_connect (G_OBJECT (pref_dialog), "response",
+    g_signal_connect (pref_dialog, "response",
 		      G_CALLBACK (pref_dialog_response), NULL);
-    g_signal_connect (G_OBJECT (pref_dialog), "delete_event",
+    g_signal_connect (pref_dialog, "delete-event",
 		      G_CALLBACK (gtk_widget_hide), NULL);
 
     gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
@@ -1472,7 +1470,7 @@ game_props_callback (void)
     gtk_table_attach_defaults (GTK_TABLE (table), l, 0, 1, 0, 1);
 
     omenu = fill_menu ();
-    g_signal_connect (G_OBJECT (omenu), "changed",
+    g_signal_connect (omenu, "changed",
 		      G_CALLBACK (set_selection), NULL);
     gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
     gtk_label_set_mnemonic_widget (GTK_LABEL (l), omenu);
@@ -1485,7 +1483,7 @@ game_props_callback (void)
     {
       w = gtk_color_button_new ();
       gtk_color_button_set_color (GTK_COLOR_BUTTON (w), &backgnd.color);
-      g_signal_connect (G_OBJECT (w), "color_set",
+      g_signal_connect (w, "color-set",
 			G_CALLBACK (bg_color_callback), NULL);
     }
 
@@ -1502,7 +1500,7 @@ game_props_callback (void)
     button = gtk_radio_button_new_with_mnemonic (NULL, _("_Small"));
     if (game_size == SMALL)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (G_OBJECT (button), "clicked",
+    g_signal_connect (button, "clicked",
 		      G_CALLBACK (size_callback), (gpointer) SMALL);
 
     gtk_container_add (GTK_CONTAINER (fv), button);
@@ -1512,7 +1510,7 @@ game_props_callback (void)
 
     if (game_size == MEDIUM)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (G_OBJECT (button), "clicked",
+    g_signal_connect (button, "clicked",
 		      G_CALLBACK (size_callback), (gpointer) MEDIUM);
     gtk_container_add (GTK_CONTAINER (fv), button);
 
@@ -1520,12 +1518,12 @@ game_props_callback (void)
       (gtk_radio_button_get_group (GTK_RADIO_BUTTON (button)), _("_Large"));
     if (game_size == LARGE)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (G_OBJECT (button), "clicked",
+    g_signal_connect (button, "clicked",
 		      G_CALLBACK (size_callback), (gpointer) LARGE);
     gtk_container_add (GTK_CONTAINER (fv), button);
 
 
-    frame = games_frame_new (_("glines|General"));
+    frame = games_frame_new (Q_("glines|General"));
     fv = gtk_vbox_new (FALSE, FALSE);
     gtk_box_set_spacing (GTK_BOX (fv), 6);
     gtk_container_add (GTK_CONTAINER (frame), fv);
@@ -1537,7 +1535,7 @@ game_props_callback (void)
       gtk_toggle_button_set_active
 	(GTK_TOGGLE_BUTTON (fast_moves_toggle_button), TRUE);
     }
-    g_signal_connect (G_OBJECT (fast_moves_toggle_button), "clicked",
+    g_signal_connect (fast_moves_toggle_button, "clicked",
 		      G_CALLBACK (set_fast_moves_callback), NULL);
 
     gtk_container_add (GTK_CONTAINER (fv), fast_moves_toggle_button);
@@ -1550,17 +1548,6 @@ game_props_callback (void)
   gtk_window_present (GTK_WINDOW (pref_dialog));
 }
 
-static gboolean
-window_resize_cb (GtkWidget * widget, GdkEventConfigure * event, void *data)
-{
-  if (!is_maximized && !is_fullscreen) {
-    gconf_client_set_int (conf_client, KEY_WIDTH, event->width, NULL);
-    gconf_client_set_int (conf_client, KEY_HEIGHT, event->height, NULL);
-  }
-
-  return FALSE;
-}
-
 static int
 game_quit_callback (GtkAction * action, gpointer data)
 {
@@ -1573,7 +1560,11 @@ game_quit_callback (GtkAction * action, gpointer data)
 static void
 game_help_callback (GtkAction * action, gpointer data)
 {
+#ifdef HAVE_GNOME
   gnome_help_display ("glines.xml", NULL, NULL);
+#else
+#warning need to port help to gtk-only!
+#endif /* HAVE_GNOME */
 }
 
 static int
@@ -1609,6 +1600,8 @@ configure_event_callback (GtkWidget * widget, GdkEventConfigure * event)
   return TRUE;
 }
 
+#ifdef HAVE_GNOME
+
 static int
 save_state (GnomeClient * client,
 	    gint phase,
@@ -1620,7 +1613,7 @@ save_state (GnomeClient * client,
   gchar *buf;
   int i;
 
-  gconf_client_set_int (conf_client, KEY_SAVED_SCORE, score, NULL);
+  games_conf_set_integer (KEY_SAVED_GROUP, KEY_SAVED_SCORE, score);
 
   buf = g_malloc (hfieldsize * vfieldsize * 4 + 1);
   for (i = 0; i < hfieldsize * vfieldsize; i++) {
@@ -1630,32 +1623,38 @@ save_state (GnomeClient * client,
     buf[i * 4 + 3] = field[i].active + 'h';
   }
   buf[hfieldsize * vfieldsize * 4] = '\0';
-  gconf_client_set_string (conf_client, KEY_SAVED_FIELD, buf, NULL);
+  games_conf_set_string (KEY_SAVED_GROUP, KEY_SAVED_FIELD, buf);
   for (i = 0; i < npieces; i++)
     buf[i] = preview[i] + 'h';
   buf[npieces] = '\0';
-  gconf_client_set_string (conf_client, KEY_SAVED_PREVIEW, buf, NULL);
+  games_conf_set_string (KEY_SAVED_GROUP, KEY_SAVED_PREVIEW, buf);
   g_free (buf);
 
   return TRUE;
 }
 
+static gint
+client_die (GnomeClient * client, gpointer client_data)
+{
+  gtk_main_quit ();
+
+  return FALSE;
+}
+
+#endif /* HAVE_GNOME */
+
 static void
 load_properties (void)
 {
   gchar *buf;
 
-  ball_filename = gconf_client_get_string (conf_client, KEY_BALL_THEME, NULL);
-  if (ball_filename == NULL)
-    ball_filename = g_strdup ("balls.svg");
+  ball_filename = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, DEFAULT_BALL_THEME);
 
-  move_timeout = gconf_client_get_int (conf_client, KEY_MOVE_TIMEOUT, NULL);
+  move_timeout = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, NULL);
   if (move_timeout <= 0)
     move_timeout = 100;
 
-  buf = gconf_client_get_string (conf_client, KEY_BACKGROUND_COLOR, NULL);
-  if (buf == NULL)
-    buf = g_strdup ("#000000");
+  buf = games_conf_get_string_with_default (KEY_PREFERENCES_GROUP, KEY_BACKGROUND_COLOR, "#000000"); /* FIXMEchpe? */
   set_backgnd_color (buf);
   g_free (buf);
 
@@ -1672,11 +1671,11 @@ restart (void)
    * restore the game the score has to be stored somewhere
    * and without some sort of restricted-access storage the
    * user will always be able to change it. */
-  score = gconf_client_get_int (conf_client, KEY_SAVED_SCORE, NULL);
+  score = games_conf_get_integer (KEY_SAVED_GROUP, KEY_SAVED_SCORE, NULL);
   if (score < 0)
     score = 0;
 
-  buf = gconf_client_get_string (conf_client, KEY_SAVED_FIELD, NULL);
+  buf = games_conf_get_string (KEY_SAVED_GROUP, KEY_SAVED_FIELD, NULL);
   if (buf) {
     for (i = 0; i < hfieldsize * vfieldsize; i++) {
       field[i].color = CLAMP (buf[i * 4] - 'h', 1, ncolors);
@@ -1687,7 +1686,7 @@ restart (void)
     }
     g_free (buf);
   }
-  buf = gconf_client_get_string (conf_client, KEY_SAVED_PREVIEW, NULL);
+  buf = games_conf_get_string (KEY_SAVED_GROUP, KEY_SAVED_PREVIEW, NULL);
   if (buf) {
     for (i = 0; i < npieces; i++)
       preview[i] = CLAMP (buf[i] - 'h', 1, ncolors);
@@ -1695,18 +1694,7 @@ restart (void)
   }
 }
 
-
-static gint
-client_die (GnomeClient * client, gpointer client_data)
-{
-  gtk_main_quit ();
-
-  return FALSE;
-
-}
-
-
-const GtkActionEntry actions[] = {
+static const GtkActionEntry actions[] = {
   {"GameMenu", NULL, N_("_Game")},
   {"SettingsMenu", NULL, N_("_Settings")},
   {"HelpMenu", NULL, N_("_Help")},
@@ -1773,6 +1761,7 @@ create_menus (GtkUIManager * ui_manager)
   menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 }
 
+#ifdef HAVE_GNOME
 #ifndef GNOME_CLIENT_RESTARTED
 #define GNOME_CLIENT_RESTARTED(client) \
 (GNOME_CLIENT_CONNECTED (client) && \
@@ -1780,75 +1769,15 @@ create_menus (GtkUIManager * ui_manager)
  (strcmp (gnome_client_get_id (client), \
   gnome_client_get_previous_id (client)) == 0))
 #endif /* GNOME_CLIENT_RESTARTED */
+#endif /* HAVE_GNOME */
 
 static void
-ball_theme_changed_cb (GConfClient * client,
-		       guint cnxn_id, GConfEntry * entry, gpointer user_data)
+init_config (void)
 {
-  gchar *theme_tmp = NULL;
+  g_signal_connect (games_conf_get_default (), "value-changed",
+                    G_CALLBACK (conf_value_changed_cb), NULL);
 
-  theme_tmp = gconf_client_get_string (client, KEY_BALL_THEME, NULL);
-
-  if (theme_tmp) {
-    if (strcmp (theme_tmp, ball_filename) != 0) {
-      g_free (ball_filename);
-      ball_filename = theme_tmp;
-      load_theme ();
-      refresh_screen ();
-    } else
-      g_free (theme_tmp);
-  }
-
-  /* FIXME apply in the prefs dialog GUI */
-}
-
-static void
-move_timeout_changed_cb (GConfClient * client,
-			 guint cnxn_id,
-			 GConfEntry * entry, gpointer user_data)
-{
-  gint timeout_tmp;
-
-  timeout_tmp = gconf_client_get_int (client, KEY_MOVE_TIMEOUT, NULL);
-  timeout_tmp = CLAMP (timeout_tmp, 1, 1000);
-  if (timeout_tmp != move_timeout)
-    move_timeout = timeout_tmp;
-
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fast_moves_toggle_button),
-				(move_timeout == 10));
-
-}
-
-static void
-init_config (int argc, char **argv)
-{
-  conf_client = gconf_client_get_default ();
-
-  gconf_init (argc, argv, NULL);
-
-  gconf_client_add_dir (conf_client,
-			KEY_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-  gconf_client_notify_add (conf_client,
-			   KEY_BALL_THEME,
-			   ball_theme_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   KEY_MOVE_TIMEOUT,
-			   move_timeout_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   KEY_BACKGROUND_COLOR,
-			   bg_color_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (conf_client,
-			   KEY_SIZE, size_changed_cb, NULL, NULL, NULL);
-
-
-  /* These are here because they are only loaded once. */
-  width = gconf_client_get_int (conf_client, KEY_WIDTH, NULL);
-  width = MAX (width, MIN_WIDTH);
-
-  height = gconf_client_get_int (conf_client, KEY_HEIGHT, NULL);
-  height = MAX (height, MIN_HEIGHT);
-
-  game_size = gconf_client_get_int (conf_client, KEY_SIZE, NULL);
+  game_size = games_conf_get_integer (KEY_PREFERENCES_GROUP, KEY_SIZE, NULL);
   if (game_size == UNSET)
     game_size = DEFAULT_GAME_SIZE;
 
@@ -1859,15 +1788,27 @@ init_config (int argc, char **argv)
 int
 main (int argc, char *argv[])
 {
+  GOptionContext *context;
   char *label_text;
   GtkWidget *label;
   GtkWidget *vbox, *hbox;
   GtkWidget *preview_hbox;
   GtkUIManager *ui_manager;
+  guint i;
+#ifdef HAVE_GNOME
   GnomeClient *client;
   GnomeProgram *program;
-  gboolean do_maximize, do_fullscreen;
-  int i;
+#else
+  gboolean retval;
+  GError *error = NULL;
+#endif
+
+#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
+  /* If we're going to use gnome-vfs, we need to init threads before
+   * calling any glib functions.
+   */
+  g_thread_init (NULL);
+#endif
 
   setgid_io_init ();
 
@@ -1877,40 +1818,65 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  context = g_option_context_new (NULL);
+#if GLIB_CHECK_VERSION (2, 12, 0)
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+#endif
+
+#ifdef HAVE_GNOME
   program = gnome_program_init ("glines", VERSION,
 				LIBGNOMEUI_MODULE,
 				argc, argv,
-				GNOME_PARAM_POPT_TABLE, NULL,
-				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+				GNOME_PARAM_GOPTION_CONTEXT, context,
+				GNOME_PARAM_APP_DATADIR, DATADIR, /* FIXMEchpe: this ought to use SHAREDIR !! */
+                                NULL);
+#else
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
+
+  retval = g_option_context_parse (context, &argc, &argv, &error);
+  g_option_context_free (context);
+  if (!retval) {
+    g_print ("%s", error->message);
+    g_error_free (error);
+    exit (1);
+  }
+#endif /* HAVE_GNOME */
+
+  games_conf_initialise ("GLines");
 
   highscores = games_scores_new (&scoredesc);
 
-  init_config (argc, argv);
-  do_maximize = gconf_client_get_bool (conf_client, KEY_MAXIMIZED, NULL);
-  do_fullscreen = gconf_client_get_bool (conf_client, KEY_FULLSCREEN, NULL);
+  init_config ();
 
   games_stock_init ();
-  gtk_window_set_default_icon_name ("gnome-five-or-more");
-  client = gnome_master_client ();
 
-  g_signal_connect (G_OBJECT (client), "save_yourself",
+  gtk_window_set_default_icon_name ("gnome-glines");
+
+#ifdef HAVE_GNOME
+  client = gnome_master_client ();
+  g_signal_connect (client, "save-yourself",
 		    G_CALLBACK (save_state), argv[0]);
-  g_signal_connect (G_OBJECT (client), "die", G_CALLBACK (client_die), NULL);
+  g_signal_connect (client, "die",
+                    G_CALLBACK (client_die), NULL);
 
   if (GNOME_CLIENT_RESTARTED (client))
     restart ();
   else
     reset_game ();
+#else
+  reset_game ();
+#endif /* HAVE_GNOME */
 
-  app = gnome_app_new ("glines", _("Five or More"));
-  gtk_window_set_default_size (GTK_WINDOW (app), width, height);
-  gtk_widget_set_size_request (GTK_WIDGET (app), MIN_WIDTH, MIN_HEIGHT);
+  app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
-  g_signal_connect (G_OBJECT (app), "delete_event",
+  gtk_window_set_title (GTK_WINDOW (app), _("Five or More"));
+
+  gtk_window_set_default_size (GTK_WINDOW (app), DEFAULT_WIDTH, DEFAULT_HEIGHT);
+  games_conf_add_window (GTK_WINDOW (app), NULL);
+
+  g_signal_connect (app, "delete-event",
 		    G_CALLBACK (game_quit_callback), NULL);
-  g_signal_connect_after (G_OBJECT (app), "configure_event",
-		          G_CALLBACK (window_resize_cb), NULL);
-  g_signal_connect (G_OBJECT (app), "window_state_event",
+  g_signal_connect (app, "window-state-event",
 		    G_CALLBACK (window_state_callback), NULL);
 
   statusbar = gtk_statusbar_new ();
@@ -1920,7 +1886,7 @@ main (int argc, char *argv[])
   gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
 
   vbox = gtk_vbox_new (FALSE, 0);
-  gnome_app_set_contents (GNOME_APP (app), vbox);
+  gtk_container_add (GTK_CONTAINER (app), vbox);
 
   create_menus (ui_manager);
   gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
@@ -1955,10 +1921,10 @@ main (int argc, char *argv[])
     /* Yes, this is an evil hack. */
     gtk_widget_realize (preview_widgets[i]);
   }
-  g_signal_connect (G_OBJECT (preview_widgets[0]), "configure_event",
+  g_signal_connect (preview_widgets[0], "configure-event",
 		    G_CALLBACK (preview_configure_cb), NULL);
 
-  scorelabel = gtk_label_new ("");
+  scorelabel = gtk_label_new (NULL);
 
   gtk_box_pack_end (GTK_BOX (hbox), scorelabel, FALSE, FALSE, 5);
 
@@ -1970,13 +1936,13 @@ main (int argc, char *argv[])
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 
   draw_area = gtk_drawing_area_new ();
-  g_signal_connect (G_OBJECT (draw_area), "button_press_event",
+  g_signal_connect (draw_area, "button-press-event",
 		    G_CALLBACK (button_press_event), NULL);
-  g_signal_connect (G_OBJECT (draw_area), "key_press_event",
+  g_signal_connect (draw_area, "key-press-event",
 		    G_CALLBACK (key_press_event), NULL);
-  g_signal_connect (G_OBJECT (draw_area), "configure_event",
+  g_signal_connect (draw_area, "configure-event",
 		    G_CALLBACK (configure_event_callback), NULL);
-  g_signal_connect (G_OBJECT (draw_area), "expose_event",
+  g_signal_connect (draw_area, "expose-event",
 		    G_CALLBACK (field_expose_event), NULL);
   gridframe = games_grid_frame_new (hfieldsize, vfieldsize);
   games_grid_frame_set_padding (GAMES_GRID_FRAME (gridframe), 1, 1);
@@ -1997,11 +1963,6 @@ main (int argc, char *argv[])
 
   load_properties ();
 
-  if (do_maximize)
-    gtk_window_maximize (GTK_WINDOW (app));
-  if (do_fullscreen)
-    gtk_window_fullscreen (GTK_WINDOW (app));
-
   gtk_widget_show_all (app);
 
   start_game ();
@@ -2009,11 +1970,14 @@ main (int argc, char *argv[])
   /* Enter the event loop */
   gtk_main ();
 
-  g_object_unref (conf_client);
+  if (ball_preimage)
+    g_object_unref (ball_preimage);
 
-  gnome_accelerators_sync ();
+  games_conf_shutdown ();
 
+#ifdef HAVE_GNOME
   g_object_unref (program);
+#endif /* HAVE_GNOME */
 
-  return (0);
+  return 0;
 }
