@@ -188,6 +188,7 @@ class NumberBox (gtk.Widget):
             self.queue_draw()                            
 
     def button_press_cb (self, w, e):        
+        if self.read_only: return
         if e.type == gtk.gdk._2BUTTON_PRESS:
             # ignore second click (this makes a double click in the
             # middle of a cell get us a display of the numbers, rather
@@ -257,6 +258,12 @@ class NumberBox (gtk.Widget):
             # Then add
             self.set_text_interactive(txt)
 
+    def note_changed_cb (self, w, top=False):
+        if top:
+            self.set_note_text_interactive(top_text=w.get_text())
+        else:
+            self.set_note_text_interactive(bottom_text=w.get_text())
+
     def show_note_editor (self, top=True):
         alloc = self.get_allocation()
         w = gtk.Window()
@@ -271,12 +278,9 @@ class NumberBox (gtk.Widget):
         if top: e.set_text(self.top_note_text)
         else: e.set_text(self.bottom_note_text)
         w.add(f)
-        if top:
-            e.connect('changed',lambda *args: self.set_note_text_interactive(top_text=e.get_text()))
-        else:
-            e.connect('changed',lambda *args: self.set_note_text_interactive(bottom_text=e.get_text()))
-        e.connect('focus-out-event',lambda *args: w.destroy())
-        e.connect('activate',lambda *args: w.destroy())
+        e.connect('changed', self.note_changed_cb, top)
+        e.connect('focus-out-event',lambda e, ev, w: w.destroy(), w)
+        e.connect('activate',lambda e, w: w.destroy(), w)
         x,y = self.window.get_origin()
         if top:
             w.move(x,y)
@@ -285,6 +289,11 @@ class NumberBox (gtk.Widget):
         w.show_all()
         e.grab_focus()
 
+    def number_changed_cb (self, ns, w):
+        w.destroy()
+        self.set_text_interactive('')
+        self.set_text_interactive(str(ns.get_value()))
+
     def show_number_picker (self):
         #self.number_picker_mode = True
         #return
@@ -293,19 +302,15 @@ class NumberBox (gtk.Widget):
         # and has problems with non-focus operations (i.e. hiding the main window).
         # YOU should rewrite this. Yes, you the reader :)
         w = gtk.Window()
-        
+        w.set_transient_for(self.get_toplevel())
         w.set_app_paintable(True)
         w.set_property('skip-pager-hint', True)
         w.set_property('skip-taskbar-hint', True)
         w.set_decorated(False)
         ns = NumberSelector(upper=self.upper,default=self.get_value())
-        def number_changed_cb (b):
-            w.destroy()
-            self.set_text_interactive('')
-            self.set_text_interactive(str(b.get_value()))
-        ns.connect('changed',number_changed_cb)
+        ns.connect('changed', self.number_changed_cb, w)
         w.grab_focus()
-        w.connect('focus-out-event',lambda *args: w.destroy())
+        w.connect('focus-out-event',lambda w, ev: w.destroy())
         w.add(ns)
         w.show()
         r = w.get_allocation()
@@ -1183,7 +1188,8 @@ class SudokuGameDisplay (SudokuNumberGrid, gobject.GObject):
         if not e: return
 	filled = self.grid.auto_fill_for_xy(e.x,e.y)
         if filled and filled!=-1:
-            self.add_value(filled[0][0],filled[0][1],filled[1])
+            e.set_text_interactive('')
+            e.set_text_interactive(str(filled[1]))
     
     @simple_debug
     def mark_impossible_implications (self, x, y):
