@@ -23,35 +23,40 @@
 
 #include <gnome.h>
 #include "tetris.h"
+#include "blockops.h"
+#include "renderer.h"
 
-#define PREVIEW_SIZE 5
-
-class Preview
-{
+class Preview {
 public:
-	Preview();
-	
-	void updateSize();
+  Preview ();
+  ~Preview ();
 
-	GtkWidget * getWidget()	{return w;}
-
-	void enable(bool enable);
-	void previewBlock(int bnr, int brot, int bcolor);
+  GtkWidget *getWidget () {
+    return w;
+  } void enable (bool enable);
+  void previewBlock (int bnr, int brot, int bcolor);
+  void setTheme (int id);
 
 private:
-	GtkWidget * w;
-	gint width;
-	gint height;
+  GtkWidget * w;
+  gint width;
+  gint height;
 
-	int blocknr;
-	int blockrot;
-	int blockcolor;
+  int blocknr;
+  int blockrot;
+  int blockcolor;
 
-	bool enabled;
+  bool enabled;
 
-	static gint configure(GtkWidget * widget, GdkEventConfigure * event, Preview * preview);
-	static gint expose(GtkWidget *widget, GdkEventExpose *event, Preview * preview);
+  int themeID;
+  cairo_surface_t *background;
+
+  static gint configure (GtkWidget * widget, GdkEventConfigure * event,
+			 Preview * preview);
+  static gint expose (GtkWidget * widget, GdkEventExpose * event,
+		      Preview * preview);
+
+  Block **blocks;
 };
 
 #endif //__preview_h__
-
